template <typename T>
void Swap( T &Value1, T &Value2 )
{
  const T Tmp = Value1;

  Value1 = Value2;
  Value2 = Tmp;
}

template <typename T>
void Swap3( T &Value1, T &Value2 )
{
  static T Tmp;

  Tmp = Value1;
  Value1 = Value2;
  Value2 = Tmp;
}


template <typename T>
bool ArrayReverse2( T &Array[], const uint Start = 0, const uint Count = WHOLE_ARRAY )
{
  int i = (int)Start;
  int j = ((Count == WHOLE_ARRAY) ? ArraySize(Array) : (int)MathMin(ArraySize(Array), Start + Count)) - 1;

  const bool Res = i < j;

  while (i < j)
    Swap(Array[i++], Array[j--]);

  return(Res);
}

template <typename T>
bool ArrayReverse3( T &Array[], const uint Start = 0, const uint Count = WHOLE_ARRAY )
{
  int i = (int)Start;
  int j = ((Count == WHOLE_ARRAY) ? ArraySize(Array) : (int)MathMin(ArraySize(Array), Start + Count)) - 1;

  const bool Res = i < j;

  while (i < j)
    Swap3(Array[i++], Array[j--]);

  return(Res);
}

#define BENCH(A)                                                             \
  {                                                                          \
    const ulong StartTime = GetMicrosecondCount();                           \
    A;                                                                       \
    Print(#A + ":" + (string)(GetMicrosecondCount() - StartTime) + " mcs."); \
  }

template <typename T>
int Bench1( T &Array[], const int Amount = 1e2 )
{
  bool Res = false;

  for (int i = 0; i < Amount; i++)
    Res |= ArrayReverse(Array);

  return(Res);
}

template <typename T>
int Bench2( T &Array[], const int Amount = 1e2 )
{
  bool Res = false;

  for (int i = 0; i < Amount; i++)
    Res |= ArrayReverse2(Array);

  return(Res);
}

template <typename T>
int Bench3( T &Array[], const int Amount = 1e2 )
{
  bool Res = false;

  for (int i = 0; i < Amount; i++)
    Res |= ArrayReverse3(Array);

  return(Res);
}

void OnStart()
{
  MqlTick Array[];
  ArrayResize(Array, 1e6);

  for (int i = 0; i < 5; i++)
  {
    BENCH(Bench1(Array));
    BENCH(Bench2(Array));
    BENCH(Bench3(Array));

    Print("------------");
  }
}